var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// index.ts
var runtime_simulator_exports = {};
__export(runtime_simulator_exports, {
  dispatch: () => dispatch,
  initialize: () => initialize
});
module.exports = __toCommonJS(runtime_simulator_exports);

// src/controller.ts
var import_chalk = require("chalk");
var import_child_process = require("child_process");
var import_events = require("events");
var import_net = require("net");
var import_path = require("path");
var import_v8 = require("v8");
var readyListener = new import_events.EventEmitter();
var childProcess;
var port = 0;
var state = 0 /* none */;
async function initialize(serverWorkerPath) {
  state = 1 /* initializing */;
  const opts = (process.env.NODE_OPTIONS ?? "") + " --experimental-vm-modules --no-warnings --enable-source-maps";
  childProcess = (0, import_child_process.spawn)(
    process.execPath,
    [serverWorkerPath || (0, import_path.join)(__dirname, "server.js")],
    {
      env: {
        ...process.env,
        NODE_OPTIONS: opts,
        FORCE_COLOR: "1"
      }
    }
  );
  childProcess.stderr.on("data", (data) => {
    console.error((0, import_chalk.yellow)("[runtime-simulator STDERR]"), data.toString());
  });
  return new Promise((resolve, reject) => {
    const dataListener = (data) => {
      if (data.toString().startsWith("Listening on port")) {
        port = Number.parseInt(
          data.toString().substring("Listening on port".length).trim()
        );
        clearTimeout(timeoutHandle);
        childProcess.stdout.removeListener("data", dataListener);
        resolve();
      }
    };
    const timeoutHandle = setTimeout(() => {
      childProcess.stdout.removeListener("data", dataListener);
      reject(
        new Error("Failed to start the app function worker server: Timeout")
      );
    }, 8e3);
    childProcess.stdout.on("data", dataListener);
  }).then(() => {
    state = 2 /* ready */;
    childProcess.stdout.on("data", (data) => {
      console.log(data.toString().replace(/\n$/, ""));
    });
    if (readyListener.listenerCount("") > 0) {
      readyListener.emit("");
    }
    return childProcess;
  });
}
async function dispatch(task) {
  if (state === 0 /* none */) {
    throw new Error(
      "Failed to dispatch app function worker task: controller not initialized"
    );
  } else if (state === 1 /* initializing */) {
    await new Promise((resolve) => {
      readyListener.once("", () => resolve());
    });
  }
  const socket = new import_net.Socket();
  return new Promise((resolve, reject) => {
    const timeoutHandle = setTimeout(() => {
      reject(
        new Error(
          "Failed to connect to the app function worker server: Timeout"
        )
      );
    }, 8e3);
    socket.connect(port, "localhost", () => {
      clearTimeout(timeoutHandle);
      resolve();
    });
  }).then(() => {
    return new Promise((resolve, reject) => {
      const chunks = [];
      function dataListener(chunk) {
        chunks.push(chunk);
      }
      socket.on("data", dataListener);
      socket.once("close", () => {
        socket.removeListener("data", dataListener);
        const rpcResponse = (0, import_v8.deserialize)(
          Buffer.concat(chunks)
        );
        if (Object.prototype.hasOwnProperty.call(rpcResponse, "result")) {
          resolve(rpcResponse.result);
        } else {
          reject((0, import_v8.deserialize)(rpcResponse.error));
        }
      });
      try {
        const data = (0, import_v8.serialize)(task);
        socket.write(
          Buffer.concat([
            // Add the content length
            Buffer.from(data.byteLength.toString()),
            data
          ]),
          (error) => {
            if (error) {
              reject(error);
            }
          }
        );
      } catch (error) {
        reject(error);
      }
    });
  }).finally(() => {
    if (socket.readable) {
      socket.end();
    }
  });
}
process.once("exit", () => {
  if (childProcess) {
    childProcess.kill();
  }
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  dispatch,
  initialize
});
/**
 * @license
 * Copyright 2022 Dynatrace LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
//# sourceMappingURL=index.js.map
